@@ -2,7 +2,7 @@
 import logging
 
 
-def get_module_logger(mod_name) -> logging.Logger:
+def get_module_logger(mod_name, logging_level) -> logging.Logger:
     """
     To use this, do logger = get_module_logger(__name__)
     """
@@ -11,5 +11,5 @@ def get_module_logger(mod_name) -> logging.Logger:
     formatter = logging.Formatter("%(asctime)s %(name)-12s %(levelname)-8s %(message)s")
     handler.setFormatter(formatter)
     logger.addHandler(handler)
-    logger.setLevel(logging.INFO)
+    logger.setLevel(logging_level)
     return logger
